@@ -1,11 +1,6 @@
 ### consensus transmission tree from phybreak-object ###
 
 
-### several methods available, default is 'count' = counting infectors without resolving cycles or multiple/no index cases.
-### Other options are 'edmunds' = counting infectors with resolving cycles and multiple/no index cases; 'mpc' = maximum parent
-### credibility, selecting the sampled posterior tree with best support (summed count of infectors per host); 'mtcc' = maximum
-### transmission cluster credibility, selecting the sampled posterior tree with best support (summed count of clusters, where
-### cluster = host + all progeny) calls: .transtreecount .transtreeedmonds .mpcinfector .mtcctree ##C++ get.phylo
 
 
 #' Create a consensus transmission tree.
@@ -19,8 +14,10 @@
 #'     infector in the posterior distribution.
 #'   \item \code{"edmonds"} starts from the most frequent infector (method \code{"count"}), multiple roots and 
 #'     cycles are removed by selecting one by one the next most frequent option that minimizes the loss in support 
-#'     (\href{https://en.wikipedia.org/wiki/Edmonds\%27_algorithm}{Edmonds' algorithm}). Support is measured by the frequency of the infector in the posterior distribution.
-#'   \item{"mpc"} gives the maximum parent credibility tree as described in \href{http://journals.plos.org/ploscompbiol/article?id=10.1371/journal.pcbi.1004613}{Hall et al (2015)}. This is the tree 
+#'     (\href{https://en.wikipedia.org/wiki/Edmonds'_algorithm}{Edmonds' algorithm}). Support is measured by the 
+#'     frequency of the infector in the posterior distribution.
+#'   \item{"mpc"} gives the maximum parent credibility tree as described in 
+#'     \href{http://dx.doi.org/10.1371/journal.pcbi.1004613}{Hall et al (2015)}. This is the tree 
 #'     in the set of posterior samples that has maximum support = product of frequencies among all posterior samples. 
 #'     Support is measured by the frequency of the infector in the posterior distribution.
 #'   \item{"mtcc"} gives the maximum transmission cluster credibility tree. This is equivalent to the maximum clade
@@ -41,16 +38,19 @@
 #'   also the infection times in the first sampled tree with consensus tree topology are given.
 #' @param time.quantiles Used only if \code{infection.times = "all"} or \code{"infector"}.
 #' @param phylo.class Whether to return an object of class \code{"phylo"}, in which case a single tree 
-#'   (\code{"mpc"} or \code{"mtcc"}) from the posterior is returned (not with summary infection times).
+#'   (\code{"mpc"} or \code{"mtcc"}) from the posterior is returned (not with summary infection times). This option
+#'   is used by \code{\link{plotPhylo}}.
 #' @return If \code{phylo.class = FALSE}, a \code{data.frame} with per item (=host) its infector and support per 
 #'   infector (or cluster), and summary infection times. If \code{phylo.class = TRUE}, a class \code{"phylo"} object, a 
 #'   single tree (\code{"mpc"} or \code{"mtcc"}) from the posterior is returned (not with summary infection times).
 #' @author Don Klinkenberg \email{don@@xs4all.nl}
-#' @references \href{http://dx.doi.org/10.1101/069195}{Klinkenberg et al, on biorXiv}.
+#' @references \href{http://dx.doi.org/10.1371/journal.pcbi.1005495}{Klinkenberg et al. (2017)} Simultaneous 
+#'   inference of phylogenetic and transmission trees in infectious disease outbreaks. 
+#'   \emph{PLoS Comput Biol}, \strong{13}(5): e1005495.
 #' @examples 
 #' #First build a phybreak-object containing samples.
 #' simulation <- sim.phybreak(obsize = 5)
-#' MCMCstate <- phybreak(data = simulation$sequences, times = simulation$sample.times)
+#' MCMCstate <- phybreak(data = simulation)
 #' MCMCstate <- burnin.phybreak(MCMCstate, ncycles = 20)
 #' MCMCstate <- sample.phybreak(MCMCstate, nsample = 50, thin = 2)
 #' 
@@ -85,6 +85,7 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
     
     ### initialization
     samplesize <- min(samplesize, chainlength)
+    nsamples <- phybreak.object$d$nsamples
     obs <- phybreak.object$p$obs
     res <- c()
     
@@ -98,13 +99,13 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
     if (method[1] == "mpc") {
         res <- .mpcinfector(phybreak.object, samplesize, phylo.class, infection.times[1] == "infector.sd")
         if (phylo.class) 
-            return(get.phylo(phybreak.object, res, TRUE))
+            return(get.phylo(phybreak.object, samplenr = res, simmap = TRUE))
     }
     if (method[1] == "mtcc") {
-        res <- .mtcctree(phybreak.object$s$nodehosts[obs:(2 * obs - 1), (1:samplesize) + chainlength - samplesize], phybreak.object$s$nodetimes[obs:(2 * 
-            obs - 1), (1:samplesize) + chainlength - samplesize], c(obs, samplesize))
+        res <- .mtcctree(phybreak.object$s$nodehosts[nsamples:(nsamples + obs - 1), (1:samplesize) + chainlength - samplesize], 
+                         phybreak.object$s$nodetimes[nsamples:(nsamples + obs - 1), (1:samplesize) + chainlength - samplesize], c(obs, samplesize))
         if (phylo.class) 
-            return(get.phylo(phybreak.object, tail(res, 1) + chainlength - samplesize, TRUE))
+            return(get.phylo(phybreak.object, samplenr = tail(res, 1) + chainlength - samplesize, simmap = TRUE))
         res <- matrix(head(res, -1), ncol = 5)
     }
     # if(method[1] == 'cc.construct') { res <- matrix(.CCtranstreeconstruct( phybreak.object$s$nodehosts[obs:(2*obs-1),
@@ -117,10 +118,10 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
     
     ### build output infectors
     if (infector.name) {
-        infectors.out <- matrix(c("index", phybreak.object$d$names)[1 + res[, 1]], ncol = 1, dimnames = list(phybreak.object$d$names, 
+        infectors.out <- matrix(c("index", phybreak.object$d$hostnames)[1 + res[, 1]], ncol = 1, dimnames = list(phybreak.object$d$hostnames[1:obs], 
             "infector"))
     } else {
-        infectors.out <- matrix(res[, 1], ncol = 1, dimnames = list(phybreak.object$d$names, "infector"))
+        infectors.out <- matrix(res[, 1], ncol = 1, dimnames = list(phybreak.object$d$hostnames[1:obs], "infector"))
     }
     # support
     if (support[1] == "count") {
@@ -130,23 +131,30 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
     }
     # times
     if (infection.times[1] == "infector") {
-        posttimes <- phybreak.object$s$nodetimes[obs:(2 * obs - 1), (1:samplesize) + chainlength - samplesize]
-        posttimes[res[, 1] != phybreak.object$s$nodehosts[obs:(2 * obs - 1), (1:samplesize) + chainlength - samplesize]] <- NA
-        time.out <- t(matrix(apply(posttimes, 1, quantile, probs = time.quantiles, na.rm = TRUE), ncol = obs))
-        colnames(time.out) <- paste0("inf.times.Q", 100 * time.quantiles)
+        posttimes <- phybreak.object$s$nodetimes[nsamples:(nsamples + obs - 1), (1:samplesize) + chainlength - samplesize]
+        posttimes[res[, 1] != phybreak.object$s$nodehosts[nsamples:(nsamples + obs - 1), (1:samplesize) + chainlength - samplesize]] <- NA
+        time.out <- apply(posttimes, 1, quantile, probs = time.quantiles, na.rm = TRUE)
+        cnames <- paste0("inf.times.Q", 100 * time.quantiles)
     } else if (infection.times[1] == "infector.sd") {
         if (method[1] == "mpc" || method[1] == "mtcc") {
             time.out <- res[, 3:5]
-            colnames(time.out) <- paste0("inf.times.", c("mean", "sd", "mc.tree"))
+            cnames <- paste0("inf.times.", c("mean", "sd", "mc.tree"))
         } else {
             time.out <- res[, 3:4]
-            colnames(time.out) <- paste0("inf.times.", c("mean", "sd"))
+            cnames <- paste0("inf.times.", c("mean", "sd"))
         }
     } else {
-        posttimes <- phybreak.object$s$nodetimes[obs:(2 * obs - 1), (1:samplesize) + chainlength - samplesize]
-        time.out <- t(matrix(apply(posttimes, 1, quantile, probs = time.quantiles), ncol = obs))
-        colnames(time.out) <- paste0("inf.times.Q", 100 * time.quantiles)
+        posttimes <- phybreak.object$s$nodetimes[nsamples:(nsamples + obs - 1), (1:samplesize) + chainlength - samplesize]
+        time.out <- apply(posttimes, 1, quantile, probs = time.quantiles)
+        cnames <- paste0("inf.times.Q", 100 * time.quantiles)
     }
+    if(inherits(phybreak.object$d$reference.date, "Date")) {
+      time.out <- as.Date(time.out, origin = phybreak.object$d$reference.date)
+    } else {
+      time.out <- time.out + phybreak.object$d$reference.date
+    }
+    time.out <- as.data.frame(split(time.out, 1:length(cnames)))
+    colnames(time.out) <- cnames
     
     ### return the result
     return(data.frame(infectors = infectors.out, support = support.out, time.out))
@@ -158,18 +166,19 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
 .transtreecount <- function(phybreak.object, samplesize, includetimes) {
     ### initialize some constants
     chainlength <- length(phybreak.object$s$mu)
+    nsamples <- phybreak.object$d$nsamples
     obsize <- phybreak.object$p$obs
     samplerange <- (chainlength - samplesize + 1):chainlength
     
     ### get result
-    res <- t(matrix(with(phybreak.object, apply(s$nodehosts[obsize:(2 * obsize - 1), samplerange], 1, .postinfector, support = TRUE)), 
+    res <- t(matrix(with(phybreak.object, apply(s$nodehosts[nsamples:(nsamples + obsize - 1), samplerange], 1, .postinfector, support = TRUE)), 
         nrow = 2))
     
     ### get time summaries
     if (includetimes) {
-        timesums <- with(phybreak.object, rowSums(s$nodetimes[obsize:(2 * obsize - 1), samplerange] * (s$nodehosts[obsize:(2 * 
+        timesums <- with(phybreak.object, rowSums(s$nodetimes[nsamples:(nsamples + obsize - 1), samplerange] * (s$nodehosts[nsamples:(nsamples + 
             obsize - 1), samplerange] == res[, 1])))
-        timesumsqs <- with(phybreak.object, rowSums((s$nodetimes[obsize:(2 * obsize - 1), samplerange]^2) * (s$nodehosts[obsize:(2 * 
+        timesumsqs <- with(phybreak.object, rowSums((s$nodetimes[nsamples:(nsamples + obsize - 1), samplerange]^2) * (s$nodehosts[nsamples:(nsamples + 
             obsize - 1), samplerange] == res[, 1])))
         res <- cbind(res, timesums/res[, 2])
         res <- cbind(res, sqrt((timesumsqs - res[, 3]^2 * res[, 2])/(res[, 2] - 1)))
@@ -186,19 +195,22 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
 .transtreeedmonds <- function(phybreak.object, samplesize, includetimes) {
     ### initialize some constants
     chainlength <- length(phybreak.object$s$mu)
+    nsamples <- phybreak.object$d$nsamples
     obsize <- phybreak.object$p$obs
     samplerange <- (chainlength - samplesize + 1):chainlength
     
     ### obtaining the result in steps
     
     # matrix with support for each infector (row) per host (column), with 0 as maximum, and a column for the index
-    supportmatrix <- cbind(c(0, rep(-1, obsize)), apply(1 + phybreak.object$s$nodehosts[obsize:(2 * obsize - 1), samplerange], 
+    supportmatrix <- cbind(c(0, rep(-1, obsize)), apply(1 + phybreak.object$s$nodehosts[nsamples:(nsamples + obsize - 1), samplerange], 
         1, tabulate, nbins = obsize + 1))
-    supportmatrix <- supportmatrix - rep(apply(supportmatrix, 2, max), each = obsize + 1)
+    maxsupportperhost <- apply(supportmatrix, 2, max)
+    supportmatrix <- supportmatrix - rep(maxsupportperhost, each = obsize + 1)
+    maxsupportperhost <- maxsupportperhost[-1]
     
     # vector with hosts, ordered by support to be index, and vector with these supports
-    candidateindex <- order(apply(phybreak.object$s$nodehosts[obsize:(2 * obsize - 1), samplerange] == 0, 1, sum), decreasing = TRUE)
-    indexsupports <- apply(phybreak.object$s$nodehosts[obsize:(2 * obsize - 1), samplerange] == 0, 1, sum)[candidateindex]
+    candidateindex <- order(apply(phybreak.object$s$nodehosts[nsamples:(nsamples + obsize - 1), samplerange] == 0, 1, sum), decreasing = TRUE)
+    indexsupports <- apply(phybreak.object$s$nodehosts[nsamples:(nsamples + obsize - 1), samplerange] == 0, 1, sum)[candidateindex]
     
     ## index host candidate by candidate, make a tree with edmonds's algorithm after each tree, test for each remaining candidate
     ## if their support-to-be-index is smaller than the support for their infector in the last tree.  If so, stop making new
@@ -207,6 +219,7 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
     nextcandidate <- 1
     alltrees <- c()
     allsupports <- c()
+    treesupports <- c()
     # then make trees as long as bestYN == FALSE
     while (!bestYN) {
         # make copy of supportmatrix, maximally supporting the next candidate index
@@ -217,17 +230,16 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
         # make the tree
         thistree <- .edmondsiterative(suppmat, samplesize, obsize)[-1] - 1
         alltrees <- c(alltrees, thistree)
-        thissupport <- rowSums(phybreak.object$s$nodehosts[obsize:(2 * obsize - 1), samplerange] == thistree)
+        thissupport <- rowSums(phybreak.object$s$nodehosts[nsamples:(nsamples + obsize - 1), samplerange] == thistree)
         allsupports <- c(allsupports, thissupport)
+        treesupports <- c(treesupports, sum(thissupport))
         
-        # test if any unused candidate index has higher index support than infector support in last tree
-        bestYN <- TRUE
-        for (i in tail(candidateindex, -nextcandidate)) {
-            if (thissupport[i] < indexsupports[candidateindex == i]) {
-                bestYN <- FALSE
-            }
-        }
+        # test if next candidate index must result in lower tree support only by its own loss in support
         nextcandidate <- nextcandidate + 1
+        highestsupportthusfar <- max(treesupports)
+        maxsupportwithnextcandidate <- sum(maxsupportperhost[-candidateindex[nextcandidate]]) + 
+          indexsupports[candidateindex[nextcandidate]]
+        if(highestsupportthusfar > maxsupportwithnextcandidate) bestYN <- TRUE
     }
     
     # find the tree with maximum support
@@ -239,9 +251,9 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
     
     ### get time summaries
     if (includetimes) {
-        timesums <- with(phybreak.object, rowSums(s$nodetimes[obsize:(2 * obsize - 1), samplerange] * (s$nodehosts[obsize:(2 * 
+        timesums <- with(phybreak.object, rowSums(s$nodetimes[nsamples:(nsamples + obsize - 1), samplerange] * (s$nodehosts[nsamples:(nsamples + 
             obsize - 1), samplerange] == res[, 1])))
-        timesumsqs <- with(phybreak.object, rowSums((s$nodetimes[obsize:(2 * obsize - 1), samplerange]^2) * (s$nodehosts[obsize:(2 * 
+        timesumsqs <- with(phybreak.object, rowSums((s$nodetimes[nsamples:(nsamples + obsize - 1), samplerange]^2) * (s$nodehosts[nsamples:(nsamples + 
             obsize - 1), samplerange] == res[, 1])))
         res <- cbind(res, timesums/res[, 2])
         res <- cbind(res, sqrt((timesumsqs - res[, 3]^2 * res[, 2])/(res[, 2] - 1)))
@@ -257,9 +269,10 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
 .mpcinfector <- function(phybreak.object, samplesize, phylo.class = FALSE, includetimes = FALSE) {
     ### initialize some constants
     chainlength <- length(phybreak.object$s$mu)
+    nsamples <- phybreak.object$d$nsamples
     obsize <- phybreak.object$p$obs
     samplerange <- (chainlength - samplesize + 1):chainlength
-    posteriorsamples <- phybreak.object$s$nodehosts[obsize:(2 * obsize - 1), samplerange]
+    posteriorsamples <- phybreak.object$s$nodehosts[nsamples:(nsamples + obsize - 1), samplerange]
     
     ### obtaining the result in steps
     
@@ -278,13 +291,13 @@ transtree <- function(phybreak.object, method = c("count", "edmonds", "mpc", "mt
     res <- matrix(c(posteriorsamples[, besttree], allsupports[, besttree]), ncol = 2)
     
     if (includetimes) {
-        timesums <- with(phybreak.object, rowSums(s$nodetimes[obsize:(2 * obsize - 1), samplerange] * (s$nodehosts[obsize:(2 * 
+        timesums <- with(phybreak.object, rowSums(s$nodetimes[nsamples:(nsamples + obsize - 1), samplerange] * (s$nodehosts[nsamples:(nsamples + 
             obsize - 1), samplerange] == res[, 1])))
-        timesumsqs <- with(phybreak.object, rowSums((s$nodetimes[obsize:(2 * obsize - 1), samplerange]^2) * (posteriorsamples == 
+        timesumsqs <- with(phybreak.object, rowSums((s$nodetimes[nsamples:(nsamples + obsize - 1), samplerange]^2) * (posteriorsamples == 
             res[, 1])))
         res <- cbind(res, timesums/res[, 2])
         res <- cbind(res, sqrt((timesumsqs - res[, 3]^2 * res[, 2])/(res[, 2] - 1)))
-        res <- cbind(res, phybreak.object$s$nodetimes[obsize:(2 * obsize - 1), besttree + samplerange[1] - 1])
+        res <- cbind(res, phybreak.object$s$nodetimes[nsamples:(nsamples + obsize - 1), besttree + samplerange[1] - 1])
     }
     
     
